//*****************************************************//
//
// Unit: RsFileInfo.cpp
//
// Purpose: Simple component to get the file information header
//          Only using the version number information could expand this to
//          do alot more
//
// Inherits From: TComponent
//
// Project: RsMisc
//
// Author: Jeff Hiscock -- jhiscock@rapidsi.com
//
// Date: February 2000
//
// Revisions
//
// Copyright www.rapidsi.com 2000
//******************************************************//
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "RsFileInfo.h"
#pragma package(smart_init)

#include <malloc.h>
//---------------------------------------------------------------------------
// ValidCtrCheck is used to assure that the components created do not have
// any pure virtual functions.
//

static inline void ValidCtrCheck(TRsFileInfo *)
{
        new TRsFileInfo(NULL);
}
//---------------------------------------------------------------------------
__fastcall TRsFileInfo::TRsFileInfo(TComponent* Owner)
        : TComponent(Owner)
{
}
//---------------------------------------------------------------------------
namespace Rsfileinfo
{
        void __fastcall PACKAGE Register()
        {
                 TComponentClass classes[1] = {__classid(TRsFileInfo)};
                 RegisterComponents("Rapid - Misc", classes, 0);
        }
}
//---------------------------------------------------------------------------

//*******************************************************//
//
// Method: Execute
//
// Purpose: Get the file info
//
// Author: Jeff Hiscock
//
// Date: February 2000
//
// Exceptions Thrown: Standard VCL exception if the error events do
//                    not have code handlers
//
// Parameter Description:
//
// Revisions:
//
// Scope: public
//*********************************************************//
void __fastcall TRsFileInfo::Execute()
{
    //TODO: Add your source code here
   unsigned long x;

   //check file name
   if (FFileName =="")
   {
      if ( FUseExe)
         FFileName = Application->ExeName;
      else
      {
         if (FOnNoFile)
         {
            OnNoFile(this);
            return;
         }
         else
           throw Exception ("RsFileInfo - File name not set");
      }
   }

   //Get size of file version structure .. will be zero if 16 bit image or no file version
   // info available
   long fsize = GetFileVersionInfoSize(FFileName.c_str(),&x);
   void *data;
   if (fsize !=0)
      data = (void*)malloc(fsize);
   else
   {
      if (FOnReadHeaderError)
         OnReadHeaderError(this);
      else
        throw Exception ("RsFileInfo - Call to get version info size failed. Might be 16 bit image or version information not available");
      return;
   }


   bool ok = GetFileVersionInfo(FFileName.c_str(),0,fsize,data);

   if ( !ok)
   {
      if (data)
      {
         free (data);
         data = NULL;
      }

      if (FOnGetFileInfoError)
      {
          OnGetFileInfoError(this);
          return;
      }
      else
      {
         throw Exception("RsFileInfo - Call to Get File Version Info Failed");
      }
   }

   // Now get the standard VS_FIXEDFILEINFO structure which we only are using
   // the version info out of
   unsigned int info_size = sizeof(VS_FIXEDFILEINFO);

   //VS_FIXEDFILEINFO *info =(VS_FIXEDFILEINFO*)malloc(info_size);
   VS_FIXEDFILEINFO *info;
   // Version 1.0.1 it appears allocating the memory first and trying to free below
   // does not work on 98 ?? It is fine on NT 
   // not sure if this needs to be done .. but should eliminate any erroneus values
  // ZeroMemory(info,sizeof(VS_FIXEDFILEINFO));


   ok = VerQueryValue(data,"\\",(void**)&info,&info_size);

   if ( !ok)
   {
      if (data)
      {
         free(data);
         data = NULL;
      }

      if (info)
      {
         free(info);
         info = NULL;
      }

      if (FOnGetFileInfoError)
      {
          OnGetFileInfoError(this);
          return;
      }
      else
      {
         throw Exception("RsFileInfo - Call to VerQueryValue Failed");
      }

   }

   // These temps are not really needed but make things a little more readable
   unsigned int ms = info->dwFileVersionMS;
   unsigned int ls = info->dwFileVersionLS;

   FBuildNumber= LOWORD(ls);
   FReleaseNumber = HIWORD(ls);
   FMajorVersion = HIWORD(ms);
   FMinorVersion = LOWORD(ms);

   // format the string
   FVersionString =FMajorVersion + "." + FMinorVersion + "." + FReleaseNumber + "." +
                   FBuildNumber;
   FSimpleVersion = FMajorVersion + "." + FMinorVersion;




   free (data);
   data = NULL;

//   free (info);
 //  info = NULL;

}
